import java.awt.event.*;
import java.awt.*;
import java.lang.Float;
import javax.swing.*;
import java.util.*;

/**
 * Panel (JPanel).
 * Almacena todas las componentes del modelo electronico.
 * Tambien se encarga de identificar la componente seleccionada,
 * recuerda que accion (trasladar o rotar) esta activada.
 * Dirige (coordina) las acciones que debe realizar cada componente
 * (crear, trasladar, rotar).
 * Ademas revisa posibles conexiones entre pines proximos.
 * @author Carlos Cortes
 */
class Panel extends JPanel
{
    /**
     */
    public Panel()
    {
        compList = new ArrayList < Componente > ();
        actual   = NINGUNO;
        anterior = NINGUNO;
        compLocked = false;
        
        addMouseListener(new MouseHandler());
        addMouseMotionListener(new MouseMotionHandler());
    }
    
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
        Componente comp;
        
        /* dibuja todos los componentes, en orden inverso
           para no tapar el componente seleccionado */
        int c, e;
        for (c = compList.size() - 1; c >= 0; c--)
        {
            comp = compList.get(c);
            
            if (accion != null && actual == c)
                g2.setColor(Color.RED);
            
            for (e = 0; e < comp.numShapes(); e++)
                g2.draw(comp.getShape(e));
            
            if (comp.tieneValor())
                g2.drawString(String.valueOf(comp.getValor()),
                              (int)(comp.getPosTexto().getX()),
                              (int)(comp.getPosTexto().getY()));
            
            if (actual == c)
                g2.setColor(Color.BLACK);
        }
    }
    
    /**
     * Recuerda que accion del menu Editar fue seleccionada.
     * @param ac Accion activa.
     */
    public void setAccion(String ac)
    {
        this.accion = ac;
    }
    
    /**
     * @return La cantidad de Componentes almacenados en el Panel.
     */
    public int numComp()
    {
        return compList.size();
    }
    
    /**
     * Agrega un Componente electronico al panel.
     * @param comp Componente a agregar.
     */
    public void add(Componente comp)
    {
        accion = null;
        compList.add(comp);
    }
    
    /**
     * Entrega la ubicacion del primer componente de la lista
     * (el mas viejo) que contenga al punto 'p' en su area activa.
     * @param p Punto a revisar.
     * @return Indice del Componente que contiene el punto.
     */
    public int find(Point p)
    {
        int i;
        Componente comp;
        for (i = 0; i < compList.size(); i++)
        {
            comp = compList.get(i);
            if (comp.getBounds().contains(p))
                return i;
        }
        return NINGUNO;
    }
    
    /**
     * Entrega un objeto que describe una conexion entre dos pines,
     * si estos estan suficientemente proximos.
     * @param comp1 Componente 1
     * @param p1 Pin del componente 1
     * @param comp2 Componente 2
     * @param p2 Pin del componente 2
     * @return Un objeto que describe una conexion entre dos pines.
     */
    private Conexion intentarConexion(Componente comp1, int p1,
                                      Componente comp2, int p2)
    {
        Point pin1;
        Point pin2;
        
        if (compList.size() < 2)
            return null;
        
        pin1 = comp1.getPinList().get(p1);
        pin2 = comp2.getPinList().get(p2);
        if (pin1.distance(pin2) <= (double)Defs.CERCA)
            return new Conexion(comp1, p1, comp2, p2);
        else
            return null;
    }
    
    /**
     * Clase interna a Panel.
     * Encargada de procesar los movimientos del mouse.
     */
    private class MouseMotionHandler implements MouseMotionListener
    {
        public void mouseMoved(MouseEvent event)
        {
            if (!compLocked)
            {
                actual = find(event.getPoint());
                if (actual != anterior)
                {
                    repaint();
                    anterior = actual;
                }
            }
        }
        
        public void mouseDragged(MouseEvent event)
        {
            Point mpos;
            ArrayList < Point > pinListActual;
            ArrayList < Point > pinListOtro;
            Componente compActual;
            Componente compOtro;
            Conexion conexion;
            int c, pa, po;
            
            if (accion == null) return;
            
            if (accion.equals(Defs.MOVER) && actual >= 0)
            {
                mpos = event.getPoint();
                compActual = compList.get(actual);
                pinListActual = compActual.getPinList();
                compActual.trasladar((int)mpos.getX(), (int)mpos.getY());
                
                /* deteccion de pines proximos */
                for (c = 0; c < compList.size(); c++)
                {
                    compOtro = compList.get(c);
                    pinListOtro = compOtro.getPinList();
                    if (actual != c)
                        for (po = 0; po < pinListOtro.size(); po++)
                            for (pa = 0; pa < pinListActual.size(); pa++)
                            {
                                conexion = intentarConexion(compActual, pa,
                                                            compOtro, po);
                                if (conexion != null)
                                    compActual.conectarCon(pa, compOtro, po);
                            }
                }
                /* fin deteccion */
                
                repaint();
            }
        }
    }
    
    /**
     * Clase interna a Panel.
     * Encargada de procesar el boton del mouse.
     */
    private class MouseHandler extends MouseAdapter
    {
        public void mousePressed(MouseEvent event)
        {
            if (accion == null) return;
            actual = find(event.getPoint());
            
            if (accion.equals(Defs.MOVER))
            {
                compLocked = true;
            }
            else
            if (accion.equals(Defs.ROTAR) && actual >= 0)
            {
                compList.get(actual).rotar();
                repaint();
            }
        }
        
        public void mouseReleased(MouseEvent event)
        {
            if (accion == null) return;
            if (accion.equals(Defs.MOVER))
                compLocked = false;
            repaint();
        }
    }
    
    private static final int NINGUNO = -1;
    
    private ArrayList < Componente > compList;
    private int actual, anterior;
    private String accion;
    
    /**
     * Necesario mientras se arrastran componentes, si el
     * mouse se mueve muy rapido, se des-seleccionarian
     * y dejarian de seguir al mouse.
     * @return Indica si el componente debe seguir seleccionado aun
     * cuando el puntero salga de su area activa.
     */
    private boolean compLocked;
}
